"""
   Descp: Use this file in order to isolate your app text

   Created on: 21-feb-2020

   Copyright 2020-2021 Youssef 'FRYoussef' El Faqir El Rhazoui
        <f.r.youssef@hotmail.com>
"""

TEXT: dict = {
    'abs_fail': 'Absolute majority rejection',
    'abs_pass': 'Absolute majority approval',
    'active_users_dp_id': 'daostack-dp-activeusers',
    'active_users_dp_title': 'Active holders',
    'active_users_title': 'Active reputation holders',
    'activity_title': 'Activity',
    'approval_proposal_rate_title': 'Approval proposal rate',
    'assets_title': 'Assets',
    'assets_value_dp_id': 'daostack-dp-assets',
    'assets_value_dp_title': 'Treasury',
    'assets_value_title': 'Tokens with fiat equivalence (USD)',
    'assets_novalue_title': 'Tokens without fiat equivalence',
    'boost': 'Rate of boosted proposals that were approved',
    'boost_fail': 'Boosted rejections',
    'boost_pass': 'Boosted approvals',
    'css_id_assets': 'daostack-assets',
    'css_id_activity': 'daostack-activity',
    'css_id_organization': 'daostack-organization',
    'css_id_proposal': 'daostack-proposal',
    'css_id_reputation_holders': 'daostack-reputation-holders',
    'css_id_stake': 'daostack-stake',
    'css_id_votes': 'daostack-votes',
    'css_pane_border': 'daostack-border',
    'different_stakers_title': 'Different stakers',
    'different_voters_title': 'Different voters',
    'disclaimer_activity': 'As activity we consider new proposals, votes and stakes',
    'downstakes': 'Downstakes',
    'new_proposals_dp_id': 'daostack-dp-proposals',
    'new_proposals_dp_title': 'New Proposals',
    'new_proposals_title': 'New proposals',
    'new_users_title': 'New reputation holders',
    'not_boost': 'Rate of non-boosted proposals that were rejected',
    'pane_css_prefix': 'daostack-pane',
    'proposal_boost_outcome_title': 'Closed proposal\'s outcome',
    'proposal_boost_succ_rate_title': 'Success rate of the stakes by type',
    'proposal_outcome_majority_title': 'Closed proposal\'s majority outcome',
    'proposal_title': 'Proposals',
    'proposal_total_succ_rate_title': 'Total success rate of the stakes',
    'queue_fail': 'Non-boosted rejections',
    'queue_pass': 'Non-boosted approvals',
    'rel_fail': 'Relative majority rejection',
    'rep_holder_title': 'Reputation holders',
    'rel_pass': 'Relative majority approval',
    'stake_title': 'Stakes',
    'title_active_organization': 'Number of DAOs active per month',
    'title_organization_activity': 'Total activity per month',
    'total_users_dp_id': 'daostack-dp-users',
    'total_users_dp_title': 'Holders',
    'total_users_title': 'Reputation holders',
    'total_stakes_option_title': 'Total stakes option',
    'total_stakes_title': 'Total stakes',
    'total_votes_option_title': 'Total votes option',
    'total_votes_title': 'Total votes',
    'upstakes': 'Upstakes',
    'vote_against_rate_title': 'Votes-against rate',
    'vote_for_rate_title': 'Votes-for rate',
    'vote_voters_title': 'Number of votes per voter',
    'vote_title': 'Votes',
    'votes_against': 'Votes against',
    'votes_for': 'Votes for',
    'voters_percentage_title': 'Percentage of reputation holders who vote',
}
