"""
   Descp: This file is used to store common text for all the apps

   Created on: 1-oct-2020

   Copyright 2020-2021 Youssef 'FRYoussef' El Faqir El Rhazoui
        <f.r.youssef@hotmail.com>
"""

TEXT: dict = {
    'about_us': 'About DAO-Analyzer',
    'acknowledgements': ' and chain-community projects were funded by the European Research Council and The Spanish Ministry of Science and Innovation',
    'active_users_title': 'Active reputation holders',
    'all_orgs': 'All DAOs',
    'app_title': 'DAO-Analyzer',
    'app_title_aragon': 'Aragon-Analyzer',
    'app_title_daohaus': 'DAOhaus-Analyzer',
    'app_title_daostack': 'DAOstack-Analyzer',
    'aragon': 'Aragon',
    'aragon_image_name': 'aragon.png',
    'bt_analyze': 'Analyze',
    'cc_image_name': 'cc-by.png',
    'cc_license_text': 'This site is licensed under a Creative Commons Attribution 4.0 International License',
    'cc_url': 'https://creativecommons.org/licenses/by/4.0/',
    'creation_date_format': '%b %-d, %Y',
    'css_color_app': 'app-color',
    'css_color_aragon': 'aragon-color',
    'css_color_daohaus': 'daohaus-color',
    'css_color_daostack': 'daostack-color',
    'current_version': 'Current version: {version}',
    'dao_analyzer_header_name': 'dao-analyzer_header.svg',
    'dao_analyzer_header_text': 'Explore the DAO world',
    'dao_analyzer_logo_name': 'logo_dao_analyzer.svg',
    'dao_selector_title': 'Select a DAO',
    'daohaus': 'DAOhaus',
    'daohaus_image_name': 'daohaus.png',
    'daostack': 'DAOstack',
    'daostack_image_name': 'daostack.png',
    'default_amount': 'NAN',
    'dp_title_members': 'Members',
    'dp_title_treasury': 'Treasury',
    'ecosystem': 'ecosystem',
    'ecosystem_selector_title': 'Select a platform',
    'erc_image_name': 'logo-erc.png',
    'erc_url': 'https://erc.europa.eu/',
    'filter_active_last_year_title': 'Active last year',
    'follow_us': 'Follow us on ',
    'github': 'GitHub',
    'github_icon_name': 'ico-github.svg',
    'github_url': 'https://github.com/Grasia/dao-analyzer',
    'gpl_image_name': 'logo-GPL.png',
    'gpl_license_text': 'The code is licensed under GPLv3',
    'gpl_url': 'https://www.gnu.org/licenses/gpl-3.0.en.html',
    'graph_amount': '{}: {}',
    'graph_subtitle': '{:.2f}% month over month',
    'grasia_url': 'https://grasia.fdi.ucm.es',
    'no_data': 'No data',
    'no_data_selected': 'There\'s no DAO selected',
    'not_found': 'Error 404: Page not found',
    'last_activity_format': '%b %-d, %Y',
    'last_update': 'Last update:',
    'logo_grasia_name': 'logo-grasia.png',
    'logo_ucm_name': 'logo-ucm.png',
    'network_selector_title': 'Network',
    'p2p_models': 'P2P Models',
    'p2p_models_url': 'https://p2pmodels.eu/',
    'radio_button_all_networks': 'All Networks',
    'spanish_ministry_image_name': 'logo-ministerio.png',
    'spanish_ministry_url': 'https://www.ciencia.gob.es/',
    'stat_members_created_proposals': ' of members have created a proposal',
    'stat_members_created_proposals_novalue': 'No members have created a proposal',
    'stat_members_ever_voted': ' of members have ever voted',
    'stat_members_ever_voted_novalue': 'No members have ever voted',
    'this_month': 'This month',
    'ucm_url': 'https://www.ucm.es/',
    # 'unknown_dao_name': '———', # Em dash
    'unknown_dao_name': 'no name given',
    'url_about': 'about',
    'url_aragon': 'aragon',
    'url_daostack': 'daostack',
    'url_daohaus': 'daohaus',
}
