from .bar_figure import BarFigure
from .calplot import CalFigure
from .double_scatter_figure import DoubleScatterFigure
from .figure import Figure
from .multi_bar_figure import MultiBarFigure
from .treemap_figure import TreemapFigure

__all__ = [
    'Figure',
    'BarFigure',
    'CalFigure',
    'DoubleScatterFigure',
    'MultiBarFigure',
    'TreemapFigure',
]